@@ -31,7 +31,7 @@ import datetime
 import time
 import collections
 
-VERSION = (0, 3, 1)
+VERSION = (0, 3, 2)
 __version__ = '%s.%s.%s' % VERSION
 apilevel = '2.0'
 threadsafety = 1
@@ -41,12 +41,8 @@ PY2 = sys.version_info[0] == 2
 
 DEBUG = False
 
-def DEBUG_OUTPUT(*argv):
-    if not DEBUG:
-        return
-    for s in argv:
-        print(s, end=' ', file=sys.stderr)
-    print(file=sys.stderr)
+def DEBUG_OUTPUT(s):
+    print(s, end=' \n', file=sys.stderr)
 
 def HEX(data):
     import binascii
@@ -239,6 +235,22 @@ def _decode_column(data, oid, encoding):
         dt.replace(tzinfo=UTC())
         dt += datetime.timedelta(hours=offset)
         return dt
+    elif oid in (PG_TYPE_INTERVAL, ):
+        dt = data.split('days')
+        if len(dt) < 2:
+            days = 0
+            hours, minites, seconds = dt[0].split(':')
+        else:
+            days = int(dt[0])
+            if dt[1]:
+                hours, minites, seconds = dt[1].split(':')
+            else:
+                hours = minites = seconds = 0
+
+        hours = int(hours)
+        minites = int(minites)
+        seconds = int(seconds)
+        return datetime.timedelta(seconds=seconds, minutes=minites, hours=hours, days=days)
     elif oid in (PG_TYPE_BYTEA, ):
         assert data[:2] == u'\\x'
         hex_str = data[2:]
@@ -246,42 +258,31 @@ def _decode_column(data, oid, encoding):
         return b''.join([chr(c) for c in ia]) if PY2 else bytes(ia)
     elif oid in (PG_TYPE_TEXT, PG_TYPE_BPCHAR, PG_TYPE_VARCHAR, PG_TYPE_NAME, PG_TYPE_JSON):
         return data
-    elif oid in (PG_TYPE_UNKNOWN, PG_TYPE_PGNODETREE):
-        DEBUG_OUTPUT('NO DECODE type:', oid)
+    elif oid in (PG_TYPE_UNKNOWN, PG_TYPE_PGNODETREE, PG_TYPE_UUID, PG_TYPE_TSVECTOR):
+        if DEBUG: DEBUG_OUTPUT('NO DECODE type:%d' % (oid, ))
         return data
+    elif oid in (PG_TYPE_INT2ARRAY, PG_TYPE_INT4ARRAY):
+        return [int(i) for i in data[1:-1].split(',')]
+    elif oid in (PG_TYPE_FLOAT4ARRAY, ):
+        return [float(f) for f in data[1:-1].split(',')]
     else:
         if DEBUG:
-            raise ValueError(str(oid) + u":" + data)
-
-    # other types return as string
+            raise ValueError('Unknown oid=' + str(oid) + ":" + data)
     return data
 
 # ----------------------------------------------------------------------------
 
 def _bytes_to_bint(b):     # Read as big endian
-    r = b[0]
-    if PY2:
-        r = ord(r)
-    for i in b[1:]:
-        if PY2:
-            i = ord(i)
-        r = r * 256 + i
+    r = 0
+    for n in b:
+        r = r * 256 + (ord(n) if PY2 else n)
     return r
 
-def _bint_to_bytes(val, nbytes):    # Convert int value to big endian bytes.
-    v = abs(val)
-    b = []
-    for n in range(nbytes):
-        b.append((v >> (8*(nbytes - n - 1)) & 0xff))
-    if val < 0:
-        for i in range(nbytes):
-            b[i] = ~b[i] + 256
-        b[-1] += 1
-        for i in range(nbytes):
-            if b[nbytes -i -1] == 256:
-                b[nbytes -i -1] = 0
-                b[nbytes -i -2] += 1
-    return b''.join([chr(c) for c in b]) if PY2 else bytes(b)
+def _bint_to_bytes(val):    # Convert int value to big endian 4 bytes.
+    if PY2:
+        return chr((val >> 24) & 0xff) + chr((val >> 16) & 0xff) + chr((val >> 8) & 0xff) + chr(val & 0xff)
+    else:
+        return bytes([(val >> 24) & 0xff, (val >> 16) & 0xff, (val >> 8) & 0xff, val & 0xff])
 
 def escape_parameter(v):
     t = type(v)
@@ -338,8 +339,12 @@ TIME = DBAPITypeObject(datetime.time)
 ROWID = DBAPITypeObject()
 
 class Error(Exception):
-    def __init__(self, message):
-        self.message = message
+    def __init__(self, *args):
+        if len(args) > 0:
+            self.message = args[0]
+        else:
+            self.message = b'Database Error'
+        super(Error, self).__init__(*args)
     def __str__(self):
         return self.message
     def __repr__(self):
@@ -395,7 +400,7 @@ class Cursor(object):
         pass
 
     def execute(self, query, args=()):
-        DEBUG_OUTPUT('Cursor::execute()', query, args)
+        if DEBUG: DEBUG_OUTPUT('Cursor::execute()\t%s\t%s' % (query, args))
         self.description = []
         self._rows.clear()
         self.query = query
@@ -407,7 +412,7 @@ class Cursor(object):
         self.connection.execute(query, self)
 
     def executemany(self, query, seq_of_params):
-        DEBUG_OUTPUT('Cursor::executemany()', query, seq_of_params)
+        if DEBUG: DEBUG_OUTPUT('Cursor::executemany()\t%s\t%s' % (query, seq_of_params))
         rowcount = 0
         for params in seq_of_params:
             self.execute(query, params)
@@ -415,16 +420,19 @@ class Cursor(object):
         self._rowcount = rowcount
 
     def fetchone(self):
-        DEBUG_OUTPUT('Cursor::fetchone()')
+        if DEBUG: DEBUG_OUTPUT('Cursor::fetchone()')
         if len(self._rows):
             return self._rows.popleft()
         return None
 
     def fetchmany(self, size=1):
-        r = []
+        rs = []
         for i in range(size):
-            r.append(r.fetchone())
-        return r
+            r = self.fetchone()
+            if not r:
+                break
+            rs.append(r)
+        return rs
 
     def fetchall(self):
         r = list(self._rows)
@@ -452,7 +460,7 @@ class Cursor(object):
 
 class Connection(object):
     def __init__(self, user, password, database, host, port, timeout, use_ssl):
-        DEBUG_OUTPUT("Connection::__init__()")
+        if DEBUG: DEBUG_OUTPUT("Connection::__init__()")
         self.user = user
         self.password = password
         self.database = database
@@ -476,7 +484,7 @@ class Connection(object):
         # send code, data and PG_F_FLUSH
         self._write(b''.join([
                 chr(code) if PY2 else bytes([code]),
-                _bint_to_bytes(len(data) + 4, 4),
+                _bint_to_bytes(len(data) + 4),
                 data,
                 b'H\x00\x00\x00\x04',
             ])
@@ -489,12 +497,12 @@ class Connection(object):
             ln = _bytes_to_bint(self._read(4)) - 4
             data = self._read(ln)
             if code == PG_B_READY_FOR_QUERY:
-                DEBUG_OUTPUT("READY_FOR_QUERY:", data)
+                if DEBUG: DEBUG_OUTPUT("READY_FOR_QUERY:%s" % (data, ))
                 self.in_transaction = (data == b'T')
                 break
             elif code == PG_B_AUTHENTICATION:
                 auth_method = _bytes_to_bint(data[:4])
-                DEBUG_OUTPUT("AUTHENTICATION:auth_method=", auth_method)
+                if DEBUG: DEBUG_OUTPUT("AUTHENTICATION:auth_method=%d" % (auth_method,))
                 if auth_method == 0:      # trust
                     pass
                 elif auth_method == 5:    # md5
@@ -507,17 +515,15 @@ class Connection(object):
                     errobj = InterfaceError("Authentication method %d not supported." % (auth_method,))
             elif code == PG_B_PARAMETER_STATUS:
                 k, v, _ = data.split(b'\x00')
-                k = k.decode('ascii')
-                v = v.decode('ascii')
-                DEBUG_OUTPUT("PARAMETER_STATUS:%s=%s" % (k, v))
-                if k == 'server_encoding':
-                    self.encoding = v
+                if DEBUG: DEBUG_OUTPUT("PARAMETER_STATUS:%s=%s" % (k, v))
+                if k == b'server_encoding':
+                    self.encoding = v.decode('ascii')
             elif code == PG_B_BACKEND_KEY_DATA:
-                DEBUG_OUTPUT("BACKEND_KEY_DATA:", HEX(data))
+                if DEBUG: DEBUG_OUTPUT("BACKEND_KEY_DATA:%s" % (HEX(data), ))
             elif code == PG_B_COMMAND_COMPLETE:
                 if obj:
                     command = data[:-1].decode('ascii')
-                    DEBUG_OUTPUT("COMMAND_COMPLETE:", command)
+                    if DEBUG: DEBUG_OUTPUT("COMMAND_COMPLETE:%s" % (command, ))
                     if command == 'SHOW':
                         obj._rowcount = 1
                     else:
@@ -527,7 +533,7 @@ class Connection(object):
                                 break
             elif code == PG_B_ROW_DESCRIPTION:
                 if obj:
-                    DEBUG_OUTPUT("ROW_DESCRIPTION:", HEX(data))
+                    if DEBUG: DEBUG_OUTPUT("ROW_DESCRIPTION:%s" % (HEX(data), ))
                     count = _bytes_to_bint(data[0:2])
                     obj.description = [None] * count
                     n = 2
@@ -535,10 +541,10 @@ class Connection(object):
                     for i in range(count):
                         name = data[n:n+data[n:].find(b'\x00')]
                         n += len(name) + 1
-                        table_oid = _bytes_to_bint(data[n:n+4])
-                        table_pos = _bytes_to_bint(data[n+4:n+6])
-                        modifier = _bytes_to_bint(data[n+12:n+16]),     # modifier
-                        format = _bytes_to_bint(data[n+16:n+18]),       # format
+#                        table_oid = _bytes_to_bint(data[n:n+4])
+#                        table_pos = _bytes_to_bint(data[n+4:n+6])
+#                        modifier = _bytes_to_bint(data[n+12:n+16])
+#                        format = _bytes_to_bint(data[n+16:n+18]),
                         field = (
                             name,
                             _bytes_to_bint(data[n+6:n+10]),     # type oid
@@ -551,10 +557,10 @@ class Connection(object):
                         n += 18
                         obj.description[idx] = field
                         idx += 1
-                    DEBUG_OUTPUT('\t\t', obj.description)
+                    if DEBUG: DEBUG_OUTPUT('\t\t%s' % (obj.description))
             elif code == PG_B_DATA_ROW:
                 if obj:
-                    DEBUG_OUTPUT("DATA_ROW:", HEX(data))
+                    if DEBUG: DEBUG_OUTPUT("DATA_ROW:%s" % (HEX(data), ))
                     n = 2
                     row = []
                     while n < len(data):
@@ -569,16 +575,16 @@ class Connection(object):
                     for i in range(len(row)):
                         row[i] = _decode_column(row[i], obj.description[i][1], self.encoding)
                     obj._rows.append(tuple(row))
-                    DEBUG_OUTPUT("\t\t", row)
+                    if DEBUG: DEBUG_OUTPUT("\t\t%s" % (row, ))
             elif code == PG_B_NOTICE_RESPONSE:
-                DEBUG_OUTPUT("NOTICE_RESPONSE:", HEX(data))
+                if DEBUG: DEBUG_OUTPUT("NOTICE_RESPONSE:%s" % (HEX(data), ))
                 for s in data.split(b'\x00'):
-                    DEBUG_OUTPUT("\t\t", s.decode(self.encoding))
+                    if DEBUG: DEBUG_OUTPUT("\t\t%s" % (s.decode(self.encoding)))
             elif code == PG_B_ERROR_RESPONSE:
-                DEBUG_OUTPUT("ERROR_RESPONSE:", HEX(data))
+                if DEBUG: DEBUG_OUTPUT("ERROR_RESPONSE:%s" % (HEX(data), ))
                 err = data.split(b'\x00')
                 for b in err:
-                    DEBUG_OUTPUT("\t\t", b.decode(self.encoding))
+                    if DEBUG: DEBUG_OUTPUT("\t\t%s" % (b.decode(self.encoding), ))
                 # http://www.postgresql.org/docs/9.3/static/errcodes-appendix.html
                 errcode = err[1][1:]
                 message = errcode + b':' + err[2][1:]
@@ -589,25 +595,25 @@ class Connection(object):
                 else:
                     errobj = DatabaseError(message)
             elif code == PG_B_COPY_OUT_RESPONSE:
-                DEBUG_OUTPUT("COPY_OUT_RESPONSE:")
+                if DEBUG: DEBUG_OUTPUT("COPY_OUT_RESPONSE:")
             elif code == PG_COPY_DATA:
-                DEBUG_OUTPUT("COPY_DATA")
+                if DEBUG: DEBUG_OUTPUT("COPY_DATA")
                 obj.write(data)
             elif code == PG_COPY_DONE:
-                DEBUG_OUTPUT("COPY_DONE")
+                if DEBUG: DEBUG_OUTPUT("COPY_DONE")
             elif code == PG_B_COPY_IN_RESPONSE:
-                DEBUG_OUTPUT("COPY_IN_RESPONSE")
+                if DEBUG: DEBUG_OUTPUT("COPY_IN_RESPONSE")
                 while True:
                     buf = obj.read(8192)
                     if not buf:
                         break
                     # send PG_COPY_DATA
-                    self._write(b'd' + _bint_to_bytes(len(buf) + 4, 4))
+                    self._write(b'd' + _bint_to_bytes(len(buf) + 4))
                     self._write(buf)
                 # send PG_COPY_DONE and PG_F_SYNC
                 self._write(b'c\x00\x00\x00\x04S\x00\x00\x00\x04')
             else:
-                DEBUG_OUTPUT("SKIP:", code, ln, HEX(data))
+                if DEBUG: DEBUG_OUTPUT("SKIP:%d\t%i\t%s" % (code, ln, HEX(data)))
         if errobj:
             raise errobj
         return
@@ -636,11 +642,11 @@ class Connection(object):
     def _open(self):
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.sock.connect((self.host, self.port))
-        DEBUG_OUTPUT("socket %s:%d" % (self.host, self.port))
+        if DEBUG: DEBUG_OUTPUT("socket %s:%d" % (self.host, self.port))
         if self.use_ssl:
             import ssl
-            self._write(_bint_to_bytes(8, 4))
-            self._write(_bint_to_bytes(80877103, 4))    # SSL request
+            self._write(_bint_to_bytes(8))
+            self._write(_bint_to_bytes(80877103))    # SSL request
             if self._read(1) == b'S':
                 self.sock = ssl.wrap_socket(self.sock)
             else:
@@ -648,13 +654,13 @@ class Connection(object):
         if self.timeout is not None:
             self.sock.settimeout(float(self.timeout))
         # protocol version 3.0
-        v = _bint_to_bytes(196608, 4)
+        v = _bint_to_bytes(196608)
         v += b'user\x00' + self.user.encode('ascii') + b'\x00'
         if self.database:
             v += b'database\x00' + self.database.encode('ascii') + b'\x00'
         v += b'\x00'
 
-        self._write(_bint_to_bytes(len(v) + 4, 4) + v)
+        self._write(_bint_to_bytes(len(v) + 4) + v)
         self._process_messages(None)
 
     def is_connect(self):
@@ -666,7 +672,7 @@ class Connection(object):
     def execute(self, query, obj=None):
         if not self.in_transaction:
             self.begin()
-        DEBUG_OUTPUT('Connection::execute()', query)
+        if DEBUG: DEBUG_OUTPUT('Connection::execute()\t%s' % (query, ))
         self._send_message(
             PG_F_QUERY,
             query.encode(self.encoding) + b'\x00',
@@ -679,18 +685,18 @@ class Connection(object):
         self.autocommit = autocommit
 
     def begin(self):
-        DEBUG_OUTPUT('BEGIN')
+        if DEBUG: DEBUG_OUTPUT('BEGIN')
         self._send_message(PG_F_QUERY, b"BEGIN\x00")
         self._process_messages(None)
 
     def commit(self):
-        DEBUG_OUTPUT('COMMIT')
+        if DEBUG: DEBUG_OUTPUT('COMMIT')
         if self.sock:
             self._send_message(PG_F_QUERY, b"COMMIT\x00")
             self._process_messages(None)
 
     def rollback(self):
-        DEBUG_OUTPUT('ROLLBACK')
+        if DEBUG: DEBUG_OUTPUT('ROLLBACK')
         if self.sock:
             self._send_message(PG_F_QUERY, b"ROLLBACK\x00")
             self._process_messages(None)
@@ -700,7 +706,7 @@ class Connection(object):
         self._open()
 
     def close(self):
-        DEBUG_OUTPUT('Connection::close()')
+        if DEBUG: DEBUG_OUTPUT('Connection::close()')
         if self.sock:
             # send PG_F_TERMINATE
             self._write(b'X\x00\x00\x00\x04')
